"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Schema = void 0;
const ValidationResults_1 = require("../models/ValidationResults");
class Schema {
    constructor(schema, rules) {
        this.schema = schema;
        this.rules = rules || {
            excess: 'clean',
        };
    }
    check(input) {
        var _a, _b;
        try {
            const inputKeys = Object.keys(input);
            const expectedKeys = Object.keys(this.schema);
            if (inputKeys.length < expectedKeys.length) {
                return (0, ValidationResults_1.MISSING_KEYS_INPUT)('Missing keys');
            }
            else if (inputKeys.length > expectedKeys.length && ((_a = this.rules) === null || _a === void 0 ? void 0 : _a.excess) === 'forbid') {
                return (0, ValidationResults_1.EXCESS_KEYS_INPUT)('Schema does not allow excess keys. Excess keys: ' +
                    inputKeys.filter(key => !expectedKeys.includes(key)).join(', '));
            }
            for (const [key, type] of Object.entries(this.schema)) {
                if (input[key] === undefined) {
                    return (0, ValidationResults_1.MISSING_KEYS_INPUT)('Missing key: ' + key);
                }
                const validation = this.isOfType(input[key], type);
                if (!(validation === null || validation === void 0 ? void 0 : validation.success)) {
                    return validation;
                }
            }
            const cleanInput = Object.assign({}, input);
            if (((_b = this.rules) === null || _b === void 0 ? void 0 : _b.excess) === 'clean') {
                for (const key in cleanInput) {
                    if (!Object.keys(this.schema).includes(key)) {
                        delete cleanInput[key];
                    }
                }
            }
            return (0, ValidationResults_1.VALID_INPUT)(cleanInput);
        }
        catch (error) {
            console.debug(error);
            return (0, ValidationResults_1.INTERNAL_ERROR)('Internal error');
        }
    }
    isOfType(
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    value, expectedType) {
        try {
            if (value === undefined) {
                return (0, ValidationResults_1.WRONG_TYPE_INPUT)('Value is undefined');
            }
            if (expectedType === 'any') {
                return (0, ValidationResults_1.VALID_INPUT)(value);
            }
            else if (expectedType === 'array') {
                if (!Array.isArray(value)) {
                    return (0, ValidationResults_1.WRONG_TYPE_INPUT)('Type mismatch. Expected array, got ' + typeof value);
                }
            }
            else if (expectedType === 'null') {
                if (value !== null) {
                    return (0, ValidationResults_1.WRONG_TYPE_INPUT)('Type mismatch. Expected null, got ' + typeof value);
                }
            }
            else if (typeof value !== expectedType) {
                return (0, ValidationResults_1.WRONG_TYPE_INPUT)('Type mismatch. Expected ' + expectedType + ', got ' + typeof value);
            }
            return (0, ValidationResults_1.VALID_INPUT)(value);
        }
        catch (error) {
            return (0, ValidationResults_1.INTERNAL_ERROR)('Internal error');
        }
    }
    length() {
        return Object.keys(this.schema).length;
    }
    toJSON() {
        return this.schema;
    }
}
exports.Schema = Schema;
