import { FailedValidation, SuccessfulValidation } from './ValidationResults';
export declare type SupportedTypes = 'string' | 'number' | 'boolean' | 'object' | 'array' | 'null' | 'any';
export declare type TypeMapping = {
    string: string;
    number: number;
    boolean: boolean;
    object: Record<string, unknown>;
    array: any[];
    null: null;
    any: any;
};
export declare type SchemaDefinition = {
    [key: string]: keyof TypeMapping;
};
export declare type Input<T extends SchemaDefinition> = {
    [K in keyof T]: TypeMapping[T[K]];
};
export declare type SchemaBlueprint<T extends SchemaDefinition> = {
    check(value: Input<T>): FailedValidation | SuccessfulValidation<{
        [K in keyof T]: TypeMapping[T[K]];
    }>;
    length(): number;
    toJSON(): Record<string, unknown>;
};
declare type EXCESS_ALLOW_FLAGS = 'keep' | 'clean' | 'forbid';
export declare type SchemaRules = {
    excess?: EXCESS_ALLOW_FLAGS;
};
export {};
